/*--------------------------------------------------------------------------------------------------
//
//     File Name:   StuffItSDK.h
//
//   Description:   Interfaces for accessing the StuffIt Engine.
//
//--------------------------------------------------------------------------------------------------
//
//  Copyright (c) 1992-2006 Smith Micro Software, Inc.
//
//  This source code and specific concepts contained herein are Confidential
//  Information and Property of Smith Micro Software.
//  Distribution is prohibited without written permission of Smith Micro Software.
//
//--------------------------------------------------------------------------------------------------
//
//  Allume Systems, Inc. a division of Smith Micro Software
//  185 Westridge Drive                    (831) 761-6200
//  Watsonville, CA  95076                 (831) 761-6206 FAX
//  http://www.stuffit.com/
//
//--------------------------------------------------------------------------------------------------
//  $Id: StuffItSDK.h,v 1.18.2.1 2006/10/05 00:17:52 mjones Exp $
//------------------------------------------------------------------------------------------------*/

/*!
	\mainpage StuffIt Engine SDK
	
	The StuffIt Engine SDK consists of a suite of APIs allowing a client to access
	basic \ref compression and \ref expansion functionality from the same core technology
	that is used in Smith Micro's StuffIt Deluxe applications.
 
	Also included in the SDK is the ability for a client to perform more advanced tasks such as 
	\ref manipulation and manipulation of an archive's contents.
 
	Currently the StuffIt Engine SDK is supported on both Mac OS X (10.4 and higher) and
	Microsoft Windows (2000, XP, and Vista).
 
	\section mainpage_install Installation
	For more information on installing the StuffIt Engine SDK:
 
	- \ref installation_page_mac
	- \ref installation_page_windows
 
	\section mainpage_usage Usage
 
	- \ref usage_page
	- \ref examples_page
 
	\section mainpage_other Other Notes
 
	- \ref change_log_page	
 
	\page installation_page_mac Installing the Mac StuffIt SDK
 
	\section installation Installation
	\subsection mac_install_overview Installation Overview

	When you first open the StuffIt Engine SDK distribution, you will notice that there are two folders. One folder
	is named <b>SDK APIs</b> and the other <b>Runtime Engine</b>.

	The <b>SDK APIs</b> folder contains a stub framework library named <i>StuffIt.framework</i>. This version of the framework
	should be placed into your build system wherever it is most convenient for your projects. To get your application to build
	it is only necessary to compile and link against the SDK API framework found in this folder.

	The <b>Runtime Engine</b> folder contains the two frameworks necessary for your StuffIt Engine SDK-enabled
	application to run. It is a necessity that BOTH frameworks in the <b>Runtime Engine</b> folder be
	distributed with your application. The version of the <i>StuffIt.framework</i> in this folder contains the
	actual runtime code for the StuffIt SDK, that acts as the interaction layer between your application and
	the StuffItCore. The <i>StuffItCore.framework</i> is responsible for all of the heavy lifting, as it contains the
	plugins for the	different archive, compression, and encoding formats. The <i>StuffItCore.framework</i> can be
	customized by removing the format plugin bundles that are not needed by your product.

	\subsection mac_install_steps Installation Steps
 
	To add the StuffIt Engine SDK to your Xcode project:
 
	- Open your application's Xcode project. (e.g. MyApp.xcodeproj)
	- From the <B>Project</B> menu, select <B>Add To Project...</B>. Locate the SDK APIs folder and add its
	<i>StuffIt.framework</i> to the appropriate targets of your project. After adding the framework, you may want to
	check to ensure the <i>StuffIt.framework</i> is listed under the <b>Link Binary with Libraries</b> build phase of
	the appropriate targets.
	- In source code that will be using the StuffIt SDK API, include the <i>"StuffIt/StuffIt.h"</i> header file
	- Add your custom StuffIt code.
	- Compile and link.	
 
	When it comes time to test or distribute the StuffIt Engine SDK with your application:
 
	- Open your application's Xcode project. (e.g. MyApp.xcodeproj)
	- From the <B>Project</B> menu, select <B>Add To Project...</B>. Locate the Runtime Engine folder and add
	both <i>StuffIt.framework</i> and <i>StuffItCore.framework</i>. When the <b>Add to Targets</b> sheet appears,
	do NOT add the frameworks to any targets.
	- For the target(s) you wish to test or distribute the SDK, add a new <b>Copy Files</b> build phase. To
	create a <b>Copy Files</b> build phase, select the target and from the <b>Project</b> menu, select the sub-menu
	<b>New Build Phase</b> and then select <b>New Copy Files Build Phase</b>. In the window, set the Destination
	drop down to <b>Frameworks</b> and close the window.
	- You will now need to add the StuffIt Engine SDK frameworks to the newly created <b>Copy Files</b> build
	phases. To do that from within Xcode, select both the <i>StuffIt.framework</i> and <i>StuffItCore.framework</i> in the 
	Groups & Files tree.  With the (left) mouse button depressed, drag both frameworks to each target's <b>Copy Files</b>
	build phase in the same tree and drop by releasing the mouse button.
	- Once the frameworks have been added to the appropriate target(s), build your application. Both frameworks
	should now exist in your application bundle's <b>Frameworks</b> folder.
 
	NOTE: When distributing your application it is highly recommended that both <i>StuffIt.framework</i> and 
	<i>StuffItCore.framework</i> be placed in your bundle's <i>Contents/Frameworks</i> folder rather than
	installing it in a global location such as <i>/Library/Frameworks</i> or <i>~/Library/Frameworks</i>.
	This is to avoid possible conflicts with any installed StuffIt applications. 
 
	\subsection mac_install_notes Other Notes
 
	Historically, there have been two versions of the StuffIt SDK, A and B, the latter being the current one. You
	must use the B version as in StuffIt 12, the A version is no longer included.

	The main difference between the two is which OS libraries they are linked against:

	The B version only links against CoreServices making it suitable for executables that need to run in environments
	where a WindowServer might not be present, consequently, the B version supplies no user interface what-so-ever.
	The B version also does not support older file system types, namely the FSSpec structure as well as any type that
	relies on Carbon to function such as AEDesc.
 
	\page installation_page_windows Installing the Windows StuffIt SDK
 
	You can link your application against the SDK implicitly at run time using <b>LoadLibrary()</b> Windows API, or at load-time
	using the provided import library for explicit linking. You ONLY need to link against the StuffItEngine DLL. The
	StuffItEngine DLL will try to load all necessary dependent libraries. So the user will need to make sure the OS knows
	where to find the libraries. 
 
	When installing the SDK consider the OS's restrictions and choose a location based on the Windows standard search
	path sequence: 
 
	- Directory from which your application will be loaded. 
	- Current directory 
	- The directories that are listed in the PATH environment variable. 
	- System directory (not recommended for Windows security reasons) 
	- Windows directory (not recommended for Windows security reasons)
 
	The StuffIt SDK DLL files must reside in the same folder as the <b>plugins</b> folder. The plugins folder
	contains the individual plugin DLLs by format. So for example, zip.dll includes most of the functionality
	to compress, expand, browse, and manipulate Zip archives. Based on your license, not all plugins need to be
	distributed with your application. 
 
	To unlock the SDK sample applications, make sure the registration text file "registration.txt" contains a valid
	license number and place it in [SDK FOLDER]/win32/msvc8/bin directory along with the sample applications. To unlock
	the SDK	from your application make sure to pass a valid license number to the OpenStuffItEngine() API.
 
	The threads library pthreadVCE.dll also should be available at run time for the SDK regardless of if you use
	the SDK's multi-threading option or not. The SDK DLLs are linked dynamically with Microsoft multithreaded run-time
	libraries. This requires the MS run-time DLL files to be available at run time for the SDK as well.
	The MS run-time libraries can be found in redist folder. 
 
	It is also highly recommended to build your client application with the /MD[d] option to avoid discrepancies
	in memory allocations and other possible problems. For more information on linking with run-time libraries
	refer to the MSDN documentation.
	
	\page usage_page Using the StuffIt Engine SDK
	\section using Using the StuffIt Engine SDK
	
	In order for the SDK to be cross platform, certain data types and object representations
	needed to be abstracted. \ref strings and references to \ref fsobjects need to be converted
	from their platform specific types to their respective StuffIt SDK types.
	
	\subsection strings Strings
	
	The StringRef is used to provide strings to the SDK where necessary. A StringRef can
	be created a number in a number of ways that is dependant on the platform the SDK is 
	executing on. For instance, a c string or pascal string may be used to create a StringRef on
	Windows or MacOS, but only the MacOS supports creation of a StringRef from a CFString.
	
	Example:
	\code
	NewStringRefFromCString("hello world", &newString)
	\endcode

	See StuffItSDKStrings.h.
	
	\subsection fsobjects File System Objects
	
	An #FSObjectRef is the sole way of providing objects to the SDK that represent an item
	within the file system. Similar to #StringRef, creation of an #FSObjectRef varies
	by platform.
	
	Example:
	\code
	NewFSObjectFromFSRef(&someFileRef, NULL, &newFSObject)
	\endcode

	See StuffItSDKFiles.h.

	\subsection usage Basic Usage
	
	Using the SDK generally entails calling #OpenStuffItEngine, building the FSObjectRefs and StringRefs that reference the
	files or archives to work with, passing them to an expansion or compression routine, disposing of the items created
	and closing the engine.
	
	Example:
	\code
	EngineError		err = kStuffItErrorNoError;
	EngineRef		engineRef = NULL;
	
	err = OpenStuffItEngine(&engineRef, NULL);
	
	...do stuff...
	
	err = CloseStuffItEngine(engineRef);
	\endcode
	
	Also see \ref examples_page.
	
	When more control is necessary, then the #EngineRef returned by #OpenStuffItEngine can be configured to behave in various ways.
	For instance, by default expanding a .sitx.hqx archive will expand both the binhex (.hqx) and StuffIt X archive (.sitx) in one
	step. However, this may not always be desirable and the engine can be changed to only expand the binhex portion using the
	APIs defined in StuffItSDKOptions.h, which would look like 
	\code
	SetEngineIntegerProperty(theEngine, kEnginePropertyIDExpansionOptions, kExpandOnlyInitialArchive)
	\endcode
	
	It might be helpful to think of an EngineRef as an object and it's property data members.
	
	Similarly, when dealing with compression or encoding, formats can be customized, such as enabling encryption, by calling
	APIs in StuffItSDKFormats.h. For example, to enable encryption when creating a StuffIt X archive you would call
	\code
	SetFormatIntegerProperty(theEngine, kFormatIDStuffItX, kFormatPropertyIDCurrentEncryptionMethod, kEncryptionMethodDefault)
	\endcode
	and then start the compression.
	
	\subsection expansion Expansion
	
	The most commonly used expansion API is #ExpandOrDecodeFile.
	\code
	ExpandOrDecodeFile(engineRef, someArchive, NULL, NULL, NULL)
	\endcode
	would expand an archive referenced by someArchive into the same location that the archive was in.
	The other parameters are optional and determine destination, the result, and a password respectively.
	 
	See \ref expand example.
	
	\subsection compression Compression
	
	When compressing an archive, the two most common routines are #CompressOrEncodeFSObject and #ArchiveFSObjectList.
	\code
	CompressOrEncodeFSObject(engineRef, kFormatIDStuffItX, sourceFile, NULL, NULL, NULL)
	\endcode
	would create a StuffIt X archive in the same location as the item referenced by sourceFile. Similar to expansion, the
	destination, result, and password can all be ignored to get default behavior.
	Note: If encryption is desired, it is not enough to simply supply a passphrase as formats that provide encryption may
	have different encryption methods and the format needs to know which one to use first. Calling
	\code
	SetFormatIntegerProperty(theEngine, kFormatIDStuffItX, kFormatPropertyIDCurrentEncryptionMethod, kEncryptionMethodDefault)
	\endcode
	before calling CompressOrEncodeFSObject and supplying a passphrase would create an encrypted archive.
	
	\code
	ArchiveFSObjectList(engineRef, kFormatIDStuffItX, sourceFiles, NULL, NULL, NULL)
	\endcode
	is similar to #CompressOrEncodeFSObject except that it allows for a collection of files and folders to be
	archived.

	See \ref compress example.
	
	\subsection notifications Notifications
	
	The SDK uses notifications to provide the client with status information about the expansion or compression
	as well as other interactions that might require user feedback such as request for passwords or
	locating the next part of a segmented archive.
	
	Example:
	\code
	void Expand(EngineRef engineRef, FSObjectRef someArchive)
	{
	EngineError	err = kStuffItErrorNoError;
	
	NotificationCallback	callback = NewNotificationCallback(PasswordNotification);
	err = InstallNotification(engineRef, kQueryNotificationClass, kPasswordNotification, callback, NULL);
	
	//During expansion, any encrypted archive will trigger a password notification query
	err = ExpandOrDecodeFile(engineRef, someArchive, NULL, NULL, NULL);
	
	err = RemoveNotification(engineRef, kQueryNotificationClass, kPasswordNotification);
	DisposeNotificationCallback(callback);
	}
	
	_PASCAL_ QueryNotification PasswordNotification(NotificationClass inClass, NotificationMessage inMessage, ContextRef inContext, ContextUserData inUserData)
	{
		if (inClass != kQueryNotificationClass)
			{
			return kNotificationIgnored;
			}
		
		StringRef	password;
		
		//Get the password from the user and put it in a StringRef
		//Note: The context of this callback may not be the main thread. 
		EngineError err = NewStringRefFromCString("new password", &password);
		
		//The SDK assumes ownership of the string created from NewStringRef...
		SetDataInContext(inContext, kPasswordQueryDataTag, &password, sizeof(StringRef *));
		
		return kNotificationHandled;
	}
	\endcode
	
	Notifications are bound to the engine in which they are created, so installing a notification on an EngineRef and
	using another will not yield desirable results.
	
	See StuffItSDKNotifications.h and StuffItSDKOptions.h.

	\subsection ui User Interface
	
	This section is deprecated and no longer supported.
	
	The SDK can display user interface in four different areas, progress, password entry,
	destination selection, and return receipt handling. User Interface can be entirely
	suppressed by setting the EngineStatusCallbackProcPtr to SuppressStatusProc. See 
	StuffItSDKCallbacks.h and StuffItSDKOptions.h.
	
	As of the 11.0 version, the SDK will no longer provide any user interface. On Mac OS X, the SDK links against
	CoreServices.framework allowing it to be used from many different contexts such as command line tools and
	GUI apps.
	
	The SDK provides a notification mechanism defined in StuffItSDKNotifications.h for getting status during an operation
	or when it's necessary for information to be supplied during an expansion or compression.
	
	\subsection threads Using the StuffIt SDK from a thread
	
	As of version 10.0, the SDK can be used from within a preemptive thread. When called from such
	a thread, any user interface that the engine might display will be suppressed. Typically, a client
	would use notifications to handle any user feedback. Be aware that when using notifications, the context in
	which the notification is delivered may not be in the main thread of the executable.
	
	In general, when using the StuffIt SDK from a thread, it's a good idea to create an #EngineRef specific to that thread.
	
	\section examples Examples
	Examples of SDK Usage can be found \ref examples_page here.
	
    \page change_log_page Change Log
 
    \section changelog Change Log

    \subsection thirteen_changes 13.0 changes
 
    The 13.0 version of the StuffIt SDK added duplicate folding to the sitx format. Duplicate folding describes
	the process by which when multiple items are added to an archive, duplicate files are only compressed once.
	See format property #kFormatPropertyIDFoldDuplicates for more information.
 
    \subsection twelve_changes 12.0 changes
 
    The 12.0 version of the StuffIt SDK added specific file format compressors to the sitx format to support the
    following file types: <i>jpeg, mp3, bmp, gif, png, tiff, pdf, psd, jpeg2k, jpeg lossless, pict, ptt5</i> and <i>pxm</i>.
    See format property #kFormatPropertyIDRecompression and enum #RecompressionMasks for more information. 
 
    Also in 12.0, a zip content recompression option was added to the sitx format. This allows the ability to recompress
    a zip archive's content when it is added to a sitx archive. This results in a better overall compression ratio. On 
    expansion the zip archive is reconstructed. See the format property #kFormatPropertyIDRecompress for more information.
 
    The Carbon version of the SDK (Version A) that was deprecated in 11.0 is no longer included in the
    distribution. See the \ref installation section for more details.
 
    \subsection eleven_changes 11.0 changes
 
    The SDK no longer provides any user interface. On Mac OS X, the SDK links against <b>CoreServices.framework</b>
    allowing it to be used from many different contexts such as command line tools and GUI apps.
 
    The Carbon version of the SDK has been labeled deprecated.
 
    \subsection ten_changes 10.0 changes
 
    The SDK can be used from within a preemptive thread. When called from such a thread, any user
    interface that the engine might display will be suppressed. Typically, a client would use notifications
    to handle any user feedback. Be aware that when using notifications, the context in which the
    notification is delivered may not be in the main thread of the executable.
	
*/


/*!
	\file StuffItSDK.h
	
	\brief Interfaces for accessing opening and closing the StuffIt Engine and for determining
	version information and registration.
*/

#ifndef STUFFITSDK_H
#define STUFFITSDK_H

#ifndef STUFFITSDKCONFIG_H
#include <StuffIt/StuffItSDKConfig.h>
#endif

#ifndef STUFFITSDKTYPES_H
#include <StuffIt/StuffItSDKTypes.h>
#endif

#ifndef STUFFITSDKERRORS_H
#include <StuffIt/StuffItSDKErrors.h>
#endif

#if TARGET_RT_MAC_CFM
	#ifndef __CODEFRAGMENTS__
	#include <CodeFragments.h>
	#endif
#endif

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT
#pragma import on
#endif

/*! \brief Useful for supplying no registration.

*/
#define	kNoRegistrationNumber		nil

/*! \brief Useful version constants.
    
    This is the version of the library you know about the top byte is the version, the bottom is the
    revision. See documentation for how to use this information.
*/

enum LibraryVersion {
    kCurrentLibraryVersion                  =   0x1300,		/**< kCurrentLibraryVersion Current version of the library. */
    kFirstSupportedEngine                   =   0x0700		/**< kFirstSupportedEngine Earliest version of the StuffIt Engine that can be used with the SDK. */ 
};

/*! \brief A reference to the StuffItEngine.

	An EngineRef is returned by OpenStuffItEngine and is used
	by many of the APIs throughout the SDK. It is disposed of
	with CloseStuffItEngine.

*/
typedef struct OpaqueEngineObject *         EngineRef;

/**/
/*   Engine version checking and availability                                          */
/**/

/*!
	\brief Get the version of the StuffIt Engine SDK
	
	Determine the version of the StuffIt Engine SDK.
	
	\return Version of the StuffIt Engine SDK
*/

extern _PASCAL_  int16_t
GetStuffItEngineLibraryVersion( void );

/*! \brief Get the verion of the StuffIt Engine.
	
	Determine the major version number, minor version number, patch level and build number 
    of the StuffIt Engine.
	
	The function supercedes the older GetVersionInfo function.

	\param inEngineRef Reference obtained from OpenStuffItEngine.
	\param outMajor Pointer to uint16_t representing a major version number. Null is invalid.
	\param outMinor Pointer to uint16_t representing a minor version number. Null is invalid.
	\param outPatchLevel Pointer to uint16_t representing a patch level. Null is invalid.
	\param outBuild Pointer to uint16_t representing a build number. Null is invalid.
	\return Returns an EngineError
*/
extern EngineError 
GetStuffItEngineVersionNumber(EngineRef inEngineRef, uint16_t* outMajor, uint16_t* outMinor, uint16_t* outPatchLevel, uint16_t* outBuild);

/*! \deprecated Use GetStuffItEngineVersionNumber instead. 
	\brief Get the verion of the StuffIt Engine SDK.
	
	Determine the major version number, minor version number, patch level and build number 
    of the StuffIt Engine SDK

	Note: The Mac OS version of this function returns its values in BCD making GetStuffItEngineVersionNumber a much better choice to get
	this information.
	
	\param inEngineRef Reference obtained from OpenStuffItEngine.
	\param outMajor Pointer to uint16_t representing a major version number. Null is invalid.
	\param outMinor Pointer to uint16_t representing a minor version number. Null is invalid.
	\param outPatchLevel Pointer to uint16_t representing a patch level. Null is invalid.
	\param outBuild Pointer to uint16_t representing a build number. Null is invalid.
	\return Returns an EngineError
*/
extern _PASCAL_ EngineError 
GetVersionInfo(EngineRef inEngineRef, uint16_t* outMajor, uint16_t* outMinor, uint16_t* outPatchLevel, uint16_t* outBuild);

/*! \brief Determine if the StuffIt Engine is registered
	
	Determine if the StuffIt Engine is registered

	\return True if the StuffIt Engine is registered.
*/
extern _PASCAL_ Boolean
IsStuffItEngineRegistered( void ) WEAK_LINK;

/**/
/*   Engine Open and Close                                                             */
/**/
/*! \brief Opens the StuffIt Engine
	
	Initializes the StuffIt Engine and returns a reference used throughout
	the sdk.

	\param outEngineRef Reference returned if the engine has opened successfully. Null is invalid.
	\param registrationCode License code provided by Smith Micro
	\return Returns an EngineError
*/
extern _PASCAL_  EngineError
OpenStuffItEngine(
    EngineRef *     outEngineRef,
    const char *	registrationCode ) WEAK_LINK;

/*! \brief Closes the StuffIt Engine
	
	Closes the StuffIt Engine and releases any resources created by it.
	
	\param inEngineRef Reference obtained from OpenStuffItEngine. Null is invalid.
	\return Returns an EngineError
*/
extern _PASCAL_  EngineError
CloseStuffItEngine(
    EngineRef       inEngineRef ) WEAK_LINK;


#if TARGET_RT_MAC_CFM
    #ifdef __cplusplus
        inline Boolean StuffItEngineLibraryAvailable() { return ((GetStuffItEngineLibraryVersion != (void*)kUnresolvedCFragSymbolAddress)); }
    #else
        #define StuffItEngineLibraryAvailable()  ((GetStuffItEngineLibraryVersion != (void*)kUnresolvedCFragSymbolAddress))
    #endif
#elif TARGET_RT_MAC_MACHO
	extern _PASCAL_ Boolean StuffItEngineLibraryAvailable() WEAK_LINK;
#elif TARGET_OS_WIN32
    #ifdef __cplusplus
        inline Boolean StuffItEngineLibraryAvailable() { return true; }	// TODO: Needs to change for MachO
    #else
        #define StuffItEngineLibraryAvailable()  (true)	// TODO: Needs to change for MachO
    #endif
#else
	#error Unknown Compiler Target
#endif

#ifdef PRAGMA_IMPORT_OFF
#pragma import off
#elif PRAGMA_IMPORT
#pragma import reset
#endif

#ifdef __cplusplus
}
#endif

#endif

/*!
	\page examples_page Examples
	\section compress Compression
	
	The following examples illustrate compression and expansion. They use Mac platform specific types
	for file system representation, but that is easily replaced with other representations.
	
	
	\code
	void ArchiveFiles(const std::vector<FSRef> &fileList)
	{
		EngineRef 	engineRef;
		EngineError	err = kStuffItErrorNoError;
		
		// Open the StuffIt Engine. Passing nil for the reg code param will return an error
		// if the user does not have registered version of deluxe installed.
		err = OpenStuffItEngine(&engineRef, nil);
		
		FSObjectListRef		filesToCompress;
		
		// Create a list of file system objects for the engine to compress.
		err = NewFSObjectList(mEngineRef, &filesToCompress);
		
		for (std::vector<FSRef>::iterator index = fileList.begin(); index != fileList.end(); ++index)
			{
			err = AddFSRefToFSObjectList(engineRef, *index, filesToCompress);
			}
		
		// Create a StuffItX archive in the same location as the source files
		// 
		// Uncomment to encode the archive with binhex. The first line is the preferred mechanism
		// for accessing options but is only available in sdk 8 and later. The second line is
		// available from 7.0 on and will eventually be removed.

		//err = SetEngineFormatIDProperty(engineRef, kEnginePropertyIDPostEncodingFormat, kFormatIDBinHex);
		//err = SetPostEncodingFormat(engineRef, kFormatIDBinHex);

		err = ArchiveFSObjectList(engineRef, kFormatIDStuffItX, filesToCompress, nil, nil, nil);
		
		// Everything that starts with New... has a corresponding Dispose. Similarly, everything that
		// starts with Acquire... has a paired Release.
		err = DisposeFSObjectList(filesToCompress);
		
		err = CloseStuffItEngine(engineRef);
	}
	\endcode
	
	\section expand Expansion

	\code
	void ExpandFile(const FSRef &archive)
	{
		EngineRef 	engineRef;
		EngineError	err = kStuffItErrorNoError;
		
		// Open the StuffIt Engine. Passing nil for the reg code param will return an error
		// if the user does not have registered version of deluxe installed.
		err = OpenStuffItEngine(&engineRef, nil);
		
		FSObjectRef		archiveToExpand;
		
		// Create file system object for the engine to expand.
		err = NewFSObjectFromFSRef(archive, NULL, &archiveToExpand);

		// Expand an archive in the same location as the source
		err = ExpandOrDecodeFile(engineRef, archiveToExpand, nil, nil, nil);
		
		// Everything that starts with New... has a corresponding Dispose. Similarly, everything that
		// starts with Acquire... has a paired Release.
		err = DisposeFSObject(filesToCompress);
		
		err = CloseStuffItEngine(engineRef);
	}

	\endcode
	
	\section manipulation Archive browsing

	Archive browsing and manipulation is very similar to accessing a hierarchical file system. An archive
	always has a root folder object which can be iterated, and its contents examined.
	
	\code
	void IterateArchiveRoot(EngineRef engine, FSObjectRef archiveFile)
	{
		EngineError	err = kStuffItErrorNoError;
		
		//Before opening an archive it must be acquired and eventually released.
		ArchiveRef	archiveRef;
		err = AcquireArchive(engine, archiveFile, &archiveRef);
		
		//Open the archive. Archives can be opened in read-only or read-write.
		//If the archive is encrypted, use #OpenEncryptedArchive
		err = OpenArchive(engine, archiveRef, kFullMode);
		
		//Acquire the root of the archive. It's a folder, so its contents can be
		//iterated. As will all APIs that begin with Acquire, the object will need
		//to be released.
		ArchiveObjectRef	rootObject;
		err = AcquireRootArchiveObject(archiveRef, &rootObject);
		
		//Create an iterator for the root object.
		ArchiveIteratorRef	iteratorRef;
		err = NewArchiveIterator(rootObject, &iteratorRef);
		
		//Examine the object the iterator currently points to.
		ArchiveObjectRef	currentObject;
		err = AcquireCurrentArchiveObject(iteratorRef, &currentObject);
		
		//Loop over each object the iterator references.
		while (err != kStuffItErrorNoObjectAtThatLocation)
			{
			//Get the name of the current object
			StringRef	objectName;
			err = AcquireArchiveObjectStringProperty(currentObject, kArchiveObjectPropertyIDName, &objectName);
			
			//Dispose of the name and the current object and get the next one
			err = DisposeStringRef(objectName);
			
			err = ReleaseArchiveObject(currentObject);
			
			err = AcquireNextArchiveObject(iteratorRef, &currentObject);
			}

		//And finally, dispose and release everything
		err = DisposeArchiveIterator(iteratorRef);
		
		err = ReleaseArchiveObject(rootObject);
		
		err = CloseArchive(archiveRef);
		
		err = ReleaseArchive(archiveRef);
	}
	\endcode
*/